// Change log
/*

2_1
* Added prints when doing read and write

2_0
* Added write resolver data

1_1
* Changed prints to HEX
* Decreased interval to 65ms (85ms)

1_0
* Initial version

*/

// Include libraries
#include "mcp_can.h"
#include "SPI.h"

// Variable declaration
boolean read = true;
boolean write = true;

// I/O-PINS
const uint8_t SPI_CS_PIN = 10; // D10, CS PIN for MCP2515 CAN module

MCP_CAN CAN(SPI_CS_PIN);	// Set CS pin


/********
* SETUP *
********/
void setup()
{
	Serial.begin(115200);
	
	for(int i=0; i<20; i++)
	{
		if(CAN_OK == CAN.begin(MCP_STDEXT, CAN_500KBPS, MCP_8MHZ)) // Init CAN bus : baudrate = 500k
		{
			CAN.setMode(MCP_NORMAL);  // Set operation mode to normal so the MCP2515 sends acks to received data.
			
			/*CAN.init_Mask(0,0,0x07FF0000);                // Init first mask
			CAN.init_Filt(0,0,0x01320000);                // Init first filter
			CAN.init_Filt(1,0,0x01320000);                // Init second filter
  
			CAN.init_Mask(1,0,0x07FF0000);                // Init second mask 
			CAN.init_Filt(2,0,0x01320000);                // Init third filter
			CAN.init_Filt(3,0,0x01000000);                // Init fouth filter
			CAN.init_Filt(4,0,0x01210000);                // Init fifth filter
			CAN.init_Filt(5,0,0x01000000);                // Init sixth filter
			*/
			Serial.println("MCP2515 init ok");
			break;
		}
		else
		{
			Serial.println("MCP2515 init failed");
		}
		delay(100);
	}
}


/*******
* LOOP *
*******/
void loop()
{
  static uint32_t timeSinceLast = millis();
  readCAN();
  readSerial();
  
  if(millis() - timeSinceLast > 100) // Intervall in ms
  {
		timeSinceLast = millis();
		
		readResolverDataCAN();
		writeResolverDataCAN();
	}
}


/************************************************************************
* Checks CAN-bus buffer and if new data is available prints the message *
************************************************************************/
void readCAN()
{
	uint32_t rxId;
  uint8_t len;
  uint8_t rxBuf[8];  
  static uint32_t startTime = millis(); // Set start time
        
  if(CAN_MSGAVAIL == CAN.checkReceive()) // Check if data is coming
  {
    CAN.readMsgBuf(&rxId, &len, rxBuf); // read data, rxId: message ID, ext: flag extended ID, len: data length, rxBuf: data buf
    
    Serial.print(millis() - startTime);
		Serial.print(" ");
    Serial.print(rxId, HEX);
    Serial.print(" ");
    for(int i=0; i<len; i++)
    {
      Serial.print(rxBuf[i], HEX);
      Serial.print(" ");
    }
    Serial.println();
  }
}


/**********************************************
* Sends commands to read resolver data on CAN *
**********************************************/
void readResolverDataCAN()
{
	if(read)
	{
		return;
	}

  Serial.println("r");
	static uint8_t counter = 0;
	static uint32_t startTime = millis(); // Set start time
	uint16_t txId;
	uint8_t msg[8];
	
	if(counter == 0)
	{
		txId = 0x7D2;
		msg[0] = 0x02;
		msg[1] = 0x3E;
		msg[2] = 0x00;
		msg[3] = 0xFF;
		msg[4] = 0xFF;
		msg[5] = 0xFF;
		msg[6] = 0xFF;
		msg[7] = 0xFF;
	}
  
  if(counter == 1)
	{
		txId = 0x784;
		msg[0] = 0x02;
		msg[1] = 0x10;
		msg[2] = 0xC0;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
	
	if(counter == 2)
	{
		txId = 0x784;
		msg[0] = 0x02;
		msg[1] = 0x21;
		msg[2] = 0x01;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
  
  if(counter == 3)
	{
		txId = 0x784;
		msg[0] = 0x02;
		msg[1] = 0x21;
		msg[2] = 0x03;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
	
	if(counter == 4)
	{
		txId = 0x784;
		msg[0] = 0x02;
		msg[1] = 0x21;
		msg[2] = 0x04;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
  
  CAN.sendMsgBuf(txId, 0, 8, msg);
  
  Serial.print(millis() - startTime);
	Serial.print(" ");
  Serial.print(txId, HEX);
  Serial.print(" ");
  for(int i=0; i<8; i++)
  {
    Serial.print(msg[i], HEX);
    Serial.print(" ");
  }
  Serial.println();
  
  if(counter > 4)
  {
		read = true;
		counter = 0;
	}
	else
	{
		counter++;
	}
}


/**********************************************
* Sends commands to write resolver data on CAN *
**********************************************/
void writeResolverDataCAN()
{
	if(write)
	{
		return;
	}
	
	Serial.println("w");
	static uint8_t counter = 0;
	static uint32_t startTime = millis(); // Set start time
	uint16_t txId;
	uint8_t msg[8];
	
	if(counter == 0)
	{
		txId = 0x784;
		msg[0] = 0x02;
		msg[1] = 0x3E;
		msg[2] = 0x01;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
  
  if(counter == 1)
	{
		txId = 0x745;
		msg[0] = 0x02;
		msg[1] = 0x10;
		msg[2] = 0xAA;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
	
	if(counter == 2)
	{
		txId = 0x784;
		msg[0] = 0x03;
		msg[1] = 0x3B;
		msg[2] = 0x01;
		msg[3] = 0x7C;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
	
	if(counter == 3)
	{
		txId = 0x784;
		msg[0] = 0x04;
		msg[1] = 0x3B;
		msg[2] = 0x03;
		msg[3] = 0x00;
		msg[4] = 0x88;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
	
	if(counter == 4)
	{
		txId = 0x784;
		msg[0] = 0x04;
		msg[1] = 0x3B;
		msg[2] = 0x04;
		msg[3] = 0x00;
		msg[4] = 0x5C;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
  
  if(counter == 5)
	{
		txId = 0x784;
		msg[0] = 0x02;
		msg[1] = 0x21;
		msg[2] = 0x01;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
  
  if(counter == 6)
	{
		txId = 0x784;
		msg[0] = 0x02;
		msg[1] = 0x21;
		msg[2] = 0x03;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
	
	if(counter == 7)
	{
		txId = 0x784;
		msg[0] = 0x02;
		msg[1] = 0x21;
		msg[2] = 0x04;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
	
	if(counter == 8)
	{
		txId = 0x784;
		msg[0] = 0x02;
		msg[1] = 0x10;
		msg[2] = 0x81;
		msg[3] = 0x00;
		msg[4] = 0x00;
		msg[5] = 0x00;
		msg[6] = 0x00;
		msg[7] = 0x00;
	}
	
	if(counter == 9)
	{
		txId = 0x7D2;
		msg[0] = 0x02;
		msg[1] = 0x3E;
		msg[2] = 0x00;
		msg[3] = 0xFF;
		msg[4] = 0xFF;
		msg[5] = 0xFF;
		msg[6] = 0xFF;
		msg[7] = 0xFF;
	}
  
  CAN.sendMsgBuf(txId, 0, 8, msg);
  
  Serial.print(millis() - startTime);
	Serial.print(" ");
  Serial.print(txId, HEX);
  Serial.print(" ");
  for(int i=0; i<8; i++)
  {
    Serial.print(msg[i], HEX);
    Serial.print(" ");
  }
  Serial.println();
  
  if(counter > 9)
  {
		write = true;
		counter = 0;
	}
	else
	{
		counter++;
	}
}


void readSerial()
{
	if(Serial.available() > 0)
  {
		uint8_t readByte = Serial.read();
		if(readByte == 'r')
		{
			read = false;
		}
		if(readByte == 'w')
		{
			write = false;
		}    
	}
}
